@@ -1,10 +1,27 @@
 """Get User Input from commandline"""
-import readline
 from contextlib import contextmanager
 #
 from .qform import QuestionVisitor, QuestionForm
 
 
+try:
+    import readline
+    readline.parse_and_bind('tab:complete')
+
+except ModuleNotFoundError:
+
+    class readline:
+        """Fake readline that does nothing"""
+
+        @staticmethod
+        def set_completer(arg):
+            pass
+
+        @staticmethod
+        def parse_and_bind(arg):
+            pass
+
+
 readline.parse_and_bind('tab:complete')
 
 
@@ -144,7 +161,7 @@ class CommandlineVisitor(QuestionVisitor):
             answer = input(text).strip()  # strip is important!
         except KeyboardInterrupt:
             raise SystemExit("KeyboardInterrupt: exit program") from None
-        
+
         if any(answer == helper for helper in self._helpkeys):
             if self.display_help is False:
                 if comment is None:
@@ -190,7 +207,8 @@ class AskQuestions(QuestionForm):
         """
         self.set_answers_and_presets(config, presets, raise_error=raise_read_error)
         if ask_all is True:
-            return self._ask_impl(ask_all=ask_all, ask_defaults=ask_defaults, description=description)
+            return self._ask_impl(ask_all=ask_all, ask_defaults=ask_defaults,
+                                  description=description)
         #
         if self.is_all_set:
             return self.get_answers()
@@ -215,6 +233,8 @@ class AskQuestions(QuestionForm):
             user input
         """
         self.set_answers_and_presets(config, presets)
+        if config is not None:
+            self.write_config(config)
         return self.get_answers(check=True)
 
     def generate_input(self, filename, config=None, presets=None, ask_all=False, ask_defaults=True):
@@ -264,6 +284,7 @@ class AskQuestions(QuestionForm):
         AnswerBlock
             user input
         """
-        self.visitor.visit(self, description=description, ask_all=ask_all, ask_defaults=ask_defaults)
+        self.visitor.visit(self, description=description,
+                           ask_all=ask_all, ask_defaults=ask_defaults)
         #
         return self.get_answers(check=False)
