@@ -3,6 +3,7 @@ from collections import UserDict, UserString
 from collections.abc import Mapping
 from contextlib import contextmanager
 from io import StringIO
+import json
 #
 from .answers import AnswersBlock, SubquestionsAnswer
 from .config import ConfigParser
@@ -13,7 +14,7 @@ from .questions import QuestionASTVisitor
 from .questions import Component
 #
 from .presets import PresetGenerator
-from .validator import Validator, NOT_DEFINED, file_exists
+from .validator import Validator, NOT_DEFINED, file_exists, ListValidator
 from .validator import ValidatorErrorNotChoicesSubset, ValidatorErrorNotInChoices
 from .validator import Choices, RangeExpression
 
@@ -178,7 +179,7 @@ class LiteralBlock(_ConcreteQuestionBase):
         return True
 
     def preset(self, value, choices):
-        raise Exception(f"preset not defined for Literalblock")
+        raise Exception("preset not defined for Literalblock")
 
     def get_answer(self):
         if self._answer.is_none is True:
@@ -199,7 +200,7 @@ class ConcreteQuestion(_ConcreteQuestionBase):
     """Concrete question"""
 
     __slots__ = ("_value", "_comment", "is_subquestion_main",
-                 "question", "typ", "is_optional", "is_hidden", "is_set_to_empty")
+                 "question", "typ", "is_optional", "alias", "is_hidden", "is_set_to_empty")
 
     def __init__(self, name, question, is_subquestion=False):
         #
@@ -212,6 +213,7 @@ class ConcreteQuestion(_ConcreteQuestionBase):
         else:
             self._comment = question.comment
         #
+        self.alias = question.alias
         self.question = question.question
         self.typ = question.typ
 
@@ -224,36 +226,25 @@ class ConcreteQuestion(_ConcreteQuestionBase):
         else:
             self.is_hidden = False
 
+    @property
+    def default(self):
+        default = self._value.default
+        if default is NOT_DEFINED:
+            return None
+        return default
+
+    @property
+    def is_list_validator(self):
+        return isinstance(self._value, ListValidator)
+
     @property
     def accept_empty(self):
         return self.is_optional or self._value.get() is not NOT_DEFINED
 
-    def get_answer(self):
-        """get answer back, if is optional, return None if NOT_DEFINED"""
-        answer = self._value.get()
-        if self.is_optional is True: 
-            if self.is_set_to_empty is True or answer is NOT_DEFINED:
-                return None
-        return answer
-
     @property
     def validator(self):
         return self._value
 
-    def get_answer_as_string(self):
-        """get answer back, if is optional, return None if NOT_DEFINED"""
-        if self.is_set_to_empty is True:
-            return ''
-        return self._value.answer_as_string()
-
-    def accept(self, visitor):
-        if self.is_hidden is True:
-            return visitor.visit_concrete_question_hidden(self)
-        #
-        if isinstance(self.choices, Choices):
-            return visitor.visit_concrete_question_select(self)
-        return visitor.visit_concrete_question_input(self)
-
     @property
     def has_only_one_choice(self):
         return len(self.choices) == 1
@@ -285,11 +276,33 @@ class ConcreteQuestion(_ConcreteQuestionBase):
     def choices(self):
         return self._value.choices
 
+    def get_answer(self):
+        """get answer back, if is optional, return None if NOT_DEFINED"""
+        answer = self._value.get()
+        if self.is_optional is True:
+            if self.is_set_to_empty is True or answer is NOT_DEFINED:
+                return None
+        return answer
+
+    def get_answer_as_string(self):
+        """get answer back, if is optional, return None if NOT_DEFINED"""
+        if self.is_set_to_empty is True:
+            return ''
+        return self._value.answer_as_string()
+
+    def accept(self, visitor):
+        if self.is_hidden is True:
+            return visitor.visit_concrete_question_hidden(self)
+        #
+        if isinstance(self.choices, Choices):
+            return visitor.visit_concrete_question_select(self)
+        return visitor.visit_concrete_question_input(self)
+
     def set_answer(self, value):
         self._value.set(value)
         self.is_set = True
 
-    def preset(self, value, choices):
+    def preset(self, default, choices):
         """preset new value, choices:
         important: first update choices to ensure that default in choices!
         """
@@ -300,14 +313,14 @@ class ConcreteQuestion(_ConcreteQuestionBase):
             #
             if answer is NOT_DEFINED:
                 self.is_set = False
-        if value is not None:
-            self._value.set(value)
+        if default is not None:
+            self._value.set_default(default)
 
 
 class QuestionBlock(_QuestionsContainerBase, UserDict):
     """Store a question block"""
 
-    def __init__(self, name, concrete, blocks, qform):
+    def __init__(self, name, concrete, blocks, qform, *, comment=None):
         _QuestionsContainerBase.__init__(self, name, qform)
         #
         UserDict.__init__(self)
@@ -316,6 +329,7 @@ class QuestionBlock(_QuestionsContainerBase, UserDict):
         self.blocks = blocks
         #
         self.data = concrete
+        self.comment = comment
 
     @property
     def is_set(self):
@@ -450,13 +464,14 @@ class QuestionVisitor(ABC):
             subblock.accept(self)
 
 
-def block_not_set(block_name, keys):
+def block_error(block_name, errors):
     if block_name == '':
         txt = '\n'
     else:
         txt = f"\n[{block_name}]\n"
-    txt += "\n".join(f"{key} = NotSet" for key in keys)
-    return txt + "\n"
+    for key, error in errors.items():
+        txt += f"{key} = {error}\n"
+    return txt
 
 
 class ColtErrorAnswerNotDefined(SystemExit):
@@ -471,10 +486,10 @@ class ColtErrorAnswerNotDefined(SystemExit):
 class AnswerVisitor(QuestionVisitor):
     """Visitor to collect answers from a given qform"""
 
-    __slots__ = ('check', 'not_set')
+    __slots__ = ('check', 'error')
 
     def __init__(self):
-        self.not_set = None
+        self.error = None
         self.check = False
 
     def visit_qform(self, qform, check=False):
@@ -485,25 +500,33 @@ class AnswerVisitor(QuestionVisitor):
         ColtErrorAnswerNotDefined
             in case an answer is not defined
         """
-        self.not_set = {}
+        self.error = {}
         self.check = check
         answer = qform.form.accept(self)
         if check is True:
-            if self.not_set != {}:
-                raise ColtErrorAnswerNotDefined(self._create_exception(self.not_set))
-        self.not_set = {}
+            if len(self.error) != 0:
+                raise ColtErrorAnswerNotDefined(self._create_exception(self.error))
+        self.error = None
         return answer
 
     def visit_question_block(self, block):
         """Visit the question block and store all results in a Mapping"""
-        out = AnswersBlock({question.label: question.accept(self)
-                            for question in block.concrete.values()})
+        error = {}
+        results = {}
+        for question in block.concrete.values():
+            try:
+                res = question.accept(self)
+                results[question.label] = res
+                if res is NOT_DEFINED:
+                    error[question.label] = "NotSet"
+            except ValueError as e:
+                error[question.label] = e
+        out = AnswersBlock(results)
         #
         if self.check is True:
-            not_set = tuple(name for name, value in out.items() if value is NOT_DEFINED)
-            if len(not_set) > 0:
-                self.not_set[block.name] = not_set
-        #
+            if len(error) != 0:
+                self.error[block.name] = error
+        # update blocks
         out.update({name: block.accept(self) for name, block in block.blocks.items()})
         #
         return out
@@ -529,8 +552,8 @@ class AnswerVisitor(QuestionVisitor):
         return block.get_answer()
 
     @staticmethod
-    def _create_exception(not_set):
-        return "\n".join(block_not_set(block, values) for block, values in not_set.items())
+    def _create_exception(errors):
+        return "\n".join(block_error(block, berrors) for block, berrors in errors.items())
 
 
 class QuestionGeneratorVisitor(QuestionASTVisitor):
@@ -594,7 +617,8 @@ class QuestionGeneratorVisitor(QuestionASTVisitor):
                 # visit next item
                 question.accept(self)
             # create block
-            block = QuestionBlock(qid, self.concrete, self.blocks, self.qform)
+            block = QuestionBlock(qid, self.concrete, self.blocks,
+                                  self.qform, comment=block.comment)
         # if in main form, or within subquestions block, return the block
         if self.blocks is None:
             return block
@@ -666,7 +690,6 @@ class ErrorSettingAnswerFromFile(SystemExit):
         super().__init__(f"ErrorSettingAnswerFromFile: file = '{filename}'\n{msg}")
 
 
-
 class ErrorSettingAnswerFromDict(SystemExit):
     """Error when trying to read answers from a dict"""
 
@@ -676,17 +699,71 @@ class ErrorSettingAnswerFromDict(SystemExit):
         super().__init__(f"ErrorSettingAnswerFromDict:\n{msg}")
 
 
-class WriteAnswerVisitor(QuestionVisitor):
-    """Visitor to write the answers to a file"""
+class WriteJsonVisitor(QuestionVisitor):
+    """Visitor to write the answers to a string"""
 
-    __slots__ = ('txt')
+    __slots__ = ()
+
+    def visit_qform(self, qform, **kwargs):
+        data = qform.form.accept(self)
+        return json.dumps(data, indent=4)
+
+    def visit_question_block(self, block):
+        # first all normal questions
+        dct = {}
+        for name, question in block.concrete.items():
+            res = question.accept(self)
+            if res is not None:
+                dct[res] = res
+        #
+        for name, subblock in block.blocks.items():
+            dct[name] = subblock.accept(self)
+        return dct
+
+    def visit_concrete_question_select(self, question):
+        return question.get_answer_as_string()
+
+    def visit_concrete_question_hidden(self, question):
+        pass
+
+    def visit_concrete_question_input(self, question):
+        return question.get_answer_as_string()
+
+    def visit_literal_block(self, block):
+        answer = block.answer
+        if answer.is_none is True:
+            return
+        return answer
+
+    def visit_subquestion_block(self, block):
+        """visit subquestion blocks"""
+        answer = block.main_question.answer
+        dct = {'__answer__': answer}
+        if answer is None:
+            return {}
+        subblock = block.cases.get(answer)
+        if subblock is None:
+            return {}
+        dct[answer] = subblock.accept(self)
+        return dct
+
+
+class WriteConfigVisitor(QuestionVisitor):
+    """Visitor to write the answers to a string"""
+
+    __slots__ = ('txt',)
+
+    def __init__(self):
+        self.txt = ''
 
     def visit_qform(self, qform, **kwargs):
         self.txt = ''
         for blockname in qform.get_blocks():
             # normal blocks
             qform[blockname].accept(self)
-        return self.txt
+        txt = self.txt
+        self.txt = ''
+        return txt
 
     def visit_question_block(self, block):
         if block.name != '':
@@ -727,7 +804,7 @@ class QuestionForm(Mapping, Component):
     # visitor to generate answers
     answer_visitor = AnswerVisitor()
     # visitor to write answers to file
-    write_visitor = WriteAnswerVisitor()
+    write_visitor = WriteConfigVisitor()
     # visitor to generate question forms
     question_generator_visitor = QuestionGeneratorVisitor()
 
@@ -806,14 +883,14 @@ class QuestionForm(Mapping, Component):
             fhandle.write(self.write_visitor.visit(self))
 
     def set_answers_from_file(self, filename, raise_error=True):
-        errmsg = self._set_answers_from_file(filename)
-        if raise_error is True and errmsg is not None:
-            raise ErrorSettingAnswerFromFile(filename, errmsg)
+        error = self._set_answers_from_file(filename)
+        if raise_error is True and error.is_none() is False:
+            raise ErrorSettingAnswerFromDict(str(error))
 
     def set_answers_from_dct(self, dct, raise_error=True):
-        errmsg = self._set_answers_from_dct(dct)
-        if raise_error is True and errmsg is not None:
-            raise ErrorSettingAnswerFromDict(errmsg)
+        error = self._set_answers_from_dct(dct)
+        if raise_error is True and error.is_none() is False:
+            raise ErrorSettingAnswerFromDict(str(error))
 
     def set_answers_and_presets(self, config=None, presets=None, raise_error=True):
         """set both presets and answers"""
@@ -854,25 +931,29 @@ class QuestionForm(Mapping, Component):
     def __getitem__(self, key):
         return self.blocks[key]
 
+    def _set_literals(self, literals):
+        """set literals from literalblock """
+        for key, value in literals.items():
+            if value in (None, ''):
+                continue
+            self.literals[key].answer = value
+
     def _set_answers_from_file(self, filename):
         """Set answers from a given file"""
         #
         try:
             parsed, literals = ConfigParser.read(filename, self.literals)
         except FileNotFoundError:
-            return f"File '{filename}' not found!"
-        # set literal blocks
-        for key, value in literals.items():
-            if value in (None, ''):
-                continue
-            self.literals[key].answer = value
+            return ColtErrorMessage(f"File '{filename}' not found!")
+        #
+        self._set_literals(literals)
         #
         return self._set_answers_from_dct(parsed)
 
     def _set_answers_from_dct(self, dct):
         """Set the answers from a dictionary"""
         #
-        errstr = ""
+        error = ColtInputError()
         #
         for blockname, answers in dct.items():
             if blockname == ConfigParser.base:
@@ -885,19 +966,13 @@ class QuestionForm(Mapping, Component):
                 print(f"""Section = {blockname} unknown, maybe typo?""")
                 continue
 
-            errstr += self._set_block_answers(blockname, answers)
+            error.append(self._set_block_answers(blockname, answers))
         #
-        if errstr == "":
-            return None
-        return errstr
+        return error
 
     def _set_block_answers(self, blockname, answers):
-        if blockname != "":
-            error = f"[{blockname}]"
-        else:
-            error = ""
+        error = ColtBlockError(blockname)
 
-        errmsg = ""
         block = self.blocks[blockname]
         for key, answer in answers.items():
             if key not in block:
@@ -912,14 +987,11 @@ class QuestionForm(Mapping, Component):
             #
             try:
                 question.answer = answer
-            except ValueError:
-                errmsg += f"\n{key} = {answer}, ValueError"
+            except ValueError as e:
+                error[key] = f"{answer}, ValueError: {e}"
             except ValidatorErrorNotInChoices as err_choices:
-                errmsg += (f"\n{key} = {answer}, Wrong Choice: "
-                           f"{err_choices}")
-        if errmsg != "":
-            return error + errmsg
-        return ""
+                error[key] = f"{answer}, Wrong Choice: {err_choices}"
+        return error
 
     def _split_keys(self, name):
         block, key = split_keys(name)
@@ -928,3 +1000,62 @@ class QuestionForm(Mapping, Component):
             raise Exception("block unknown")
 
         return self.blocks[block], key
+
+
+class ColtBlockError:
+    """Class to handle error messages for setting a block"""
+
+    __slots__ = ('_errors', 'name')
+
+    def __init__(self, name):
+        self._errors = {}
+        self.name = name
+
+    def is_none(self):
+        return self._errors == {}
+
+    def __setitem__(self, key, value):
+        self._errors[key] = value
+
+    def __str__(self):
+        if self.is_none():
+            return ""
+        #
+        if self.name != "":
+            msg = f"[{self.name}]"
+        else:
+            msg = ""
+        #
+        msg += "\n".join(f"{key} = {err}" for key, err in self._errors.items())
+        #
+        return msg
+
+
+class ColtErrorMessage(str):
+    """String that behaves like a Colt Error"""
+
+    def is_none(self):
+        return self == ""
+
+
+class ColtInputError:
+    """Class to handle error messages for input setting"""
+
+    __slots__ = ('_errors', '_current')
+
+    def __init__(self):
+        self._errors = []
+        self._current = None
+
+    def __str__(self):
+        if self.is_none():
+            return ""
+
+        return "\n\n".join(f"{block}" for block in self._errors)
+
+    def is_none(self):
+        return len(self._errors) == 0
+
+    def append(self, block_error):
+        if block_error.is_none() is False:
+            self._errors.append(block_error)
