@@ -3,6 +3,7 @@ while automatically checking the type and doing error handling"""
 import os
 import ast
 from collections.abc import KeysView
+from collections import namedtuple
 #
 import numpy as np
 
@@ -10,6 +11,11 @@ import numpy as np
 __all__ = ["NOT_DEFINED", "Validator", "ValidatorErrorNotInChoices"]
 
 
+ValidatorType = namedtuple("ValidatorType", ("cls", "cases"))
+ValidatorParser = namedtuple("ValidatorParser", ("cls", "func"))
+ListInfo = namedtuple('ListInfo', ('is_list', 'nele'))
+
+
 class NoChoice:
     """Empty object that indicates that no choice are used"""
 
@@ -101,6 +107,9 @@ class RangeExpression:
                 return False
         return True
 
+    def __len__(self):
+        return 0
+
     def __str__(self):
         return self.as_str()
 
@@ -170,26 +179,26 @@ def abspath(answer):
 
 def file_exists(path):
     """check if file exists"""
-    path = abspath(path)
-    if not os.path.isfile(path):
-        raise ValueError(f"File does not exisit '{path}'")
-    return path
+    abs_path = abspath(path)
+    if not os.path.isfile(abs_path):
+        raise ValueError(f"File does not exist '{path}'")
+    return abs_path
 
 
 def folder_exists(path):
     """check if folder exists"""
-    path = abspath(path)
-    if not os.path.isdir(path):
-        raise ValueError(f"Folder does not exisit '{path}'")
-    return path
+    abs_path = abspath(path)
+    if not os.path.isdir(abs_path):
+        raise ValueError(f"Folder does not exist '{path}'")
+    return abs_path
 
 
 def non_existing_path(path):
     """check that there is nothing at the given path"""
-    path = abspath(path)
-    if os.path.exists(path):
-        raise ValueError(f"File/Folder does already exisit '{path}'")
-    return path
+    abs_path = abspath(path)
+    if os.path.exists(abs_path):
+        raise ValueError(f"File/Folder does already exist '{path}'")
+    return abs_path
 
 
 def bool_parser(answer):
@@ -330,20 +339,45 @@ class ValidatorErrorNotChoicesSubset(Exception):
     """Exception in case the new choices are not an subset of the old ones"""
 
 
-class ValidatorBase:
+class BaseValidator:
     """Base class to validator"""
 
-    __slots__ = ('_choices', '_value', '_string')
+    __slots__ = ('_choices', '_default', '_value', '_string', '_parse')
     # overwrite this method
 
-    def __init__(self, default=NOT_DEFINED, choices=None):
+    def __init__(self, parse_function, default=NOT_DEFINED, choices=None):
+        # has to be at the top
+        self._parse = parse_function
         self._string = NOT_DEFINED
         self._choices = self._set_choices(choices)
         self._value = self._set_value(default)
 
-    @staticmethod
-    def _parse(string):
-        """Abstract method to be overwritten"""
+    @property
+    def default(self):
+        """means value should not be set, is for documentation and help"""
+        return self._value
+
+    @property
+    def choices(self):
+        """Return choices"""
+        if self._choices is NO_CHOICE:
+            return None
+        return self._choices
+
+    @choices.setter
+    def choices(self, choices):
+        """  """
+        # will raise an value error if choices wrong!
+        choices = self._set_choices(choices)
+        # check that the new choices are a subset of the old ones
+        if not choices.is_subset(self._choices):
+            raise ValidatorErrorNotChoicesSubset(("cannot update choices,",
+                                                  " needs to be subset of the original ones"))
+        # overwrite existing ones
+        self._choices = choices
+        # validate choice, if existing default is not in choices, reset
+        if not self._choices.validate(self._value):
+            self._value = NOT_DEFINED
 
     def validate(self, value):
         """Parse a string and return its value,
@@ -382,141 +416,193 @@ class ValidatorBase:
         """set the value"""
         self._value = self._get_value(value)
 
-    @property
-    def choices(self):
-        """Return choices"""
-        if self._choices is NO_CHOICE:
-            return None
-        return self._choices
-
-    @choices.setter
-    def choices(self, choices):
-        """  """
-        # will raise an value error if choices wrong!
-        choices = self._set_choices(choices)
-        # check that the new choices are a subset of the old ones
-        if not choices.is_subset(self._choices):
-            raise ValidatorErrorNotChoicesSubset(("cannot update choices,",
-                                                  " needs to be subset of the original ones"))
-        # overwrite existing ones
-        self._choices = choices
-        # validate choice, if existing default is not in choices, reset
-        if not self._choices.validate(self._value):
-            self._value = NOT_DEFINED
+    def set_default(self, value):
+        self.set(value)
 
     def _set_value(self, value):
         if value is not NOT_DEFINED:
             value = self._get_value(value)
         return value
 
-    def _set_choices(self, in_choices):
+    def _set_choices(self, choices):
         """set choices"""
-        if in_choices is None:
+        if choices is None:
             return NO_CHOICE
-        return self.set_choices(in_choices)
+        return self.set_choices(choices)
 
-    def set_choices(self, in_choices):
+    def set_choices(self, choices):
         """set choices"""
-        if isinstance(in_choices, KeysView):
-            return Choices([self._parse(choice) for choice in in_choices])
+        if isinstance(choices, KeysView):
+            return Choices([self._parse(choice) for choice in choices])
         try:
-            choices = [self._parse(choice) for choice in list_parser(in_choices)]
+            choices = [self._parse(choice) for choice in list_parser(choices)]
         except ValueError:
             choices = None
         if choices is None:
-            raise ValueError(f"Choices '{in_choices}' cannot be parsed")
+            raise ValueError(f"Choices '{choices}' cannot be parsed")
         return Choices(choices)
 
     def _get_value(self, string):
+        """Sets the value"""
         value = self.validate(string)
         if not self._choices.validate(value):
             raise ValidatorErrorNotInChoices("Answer is not in choices")
+        # should already be stripped
         self._string = string
         return value
 
 
-class RangeValidator(ValidatorBase):
+class RangeValidator(BaseValidator):
     """Validator that allowes both `Choices` and RangeExpression"""
 
-    def set_choices(self, in_choices):
+    def set_choices(self, choices):
         """set choices"""
-        if isinstance(in_choices, KeysView):
-            return Choices([self._parse(choice) for choice in in_choices])
+        if isinstance(choices, KeysView):
+            return Choices([self._parse(choice) for choice in choices])
         #
         try:
-            return RangeExpression(in_choices)
+            return RangeExpression(choices)
         except ValueError:
             pass
         #
         try:
-            choices = [self._parse(choice) for choice in list_parser(in_choices)]
+            choices = [self._parse(choice) for choice in list_parser(choices)]
         except ValueError:
             choices = None
         #
         if choices is None:
-            raise ValueError(f"Choices '{in_choices}' cannot be parsed")
+            raise ValueError(f"Choices '{choices}' cannot be parsed")
         return Choices(choices)
 
 
-def create_validators(base_validators, range_validators):
-    """simple helper to generate validators from a dictionary
+class DelayedDefaultValidator(BaseValidator):
 
-    Parameters
-    ----------
-    base_validators: dict
-        all types that get the `ValidatorBase`
+    """Validator to check default correctness only at the end"""
 
-    range_validators: dict
-        all types that get the `RangeValidator`
+    def __init__(self, parse_function, default=NOT_DEFINED, choices=None):
+        super().__init__(parse_function, default=NOT_DEFINED, choices=choices)
+        self._default = default
+
+    @property
+    def default(self):
+        """means value should not be set, is for documentation and help"""
+        return self._default
+
+    def set_default(self, value):
+        self.set(value)
+        self._default = value
+
+    def get(self):
+        """Return self._value if its set or not!"""
+        if self._value is NOT_DEFINED:
+            if self._default is NOT_DEFINED:
+                return self._default
+            self.set(self._default)
+        return self._value
+
+
+class ListValidator(DelayedDefaultValidator):
+
+    """Validator for list(typ) syntax"""
+
+    def __init__(self, validator, nele, default=NOT_DEFINED):
+        self._validator = validator
+        self.nele = nele
+
+        def parse(inp):
+            return self.list_parse(inp)
+
+        super().__init__(parse, default=default, choices=None)
+
+    def list_parse(self, inp):
+        """parse function for list"""
+        if isinstance(inp, str):
+            lst = list_parser(inp)
+        else:
+            lst = inp
+
+        error = {'error': None, 'ele': []}
+
+        out = []
+        for ele in lst:
+            try:
+                out.append(self._validator.validate(ele))
+            except ValueError as err:
+                error['error'] = err
+                error['ele'].append(ele)
+
+        if error['error'] is not None:
+            raise ValueError(str(error['error']) +
+                             f" for elements [{', '.join(error['ele'])}] in [{', '.join(lst)}]")
+
+        if self.nele > 0:
+            if len(out) != self.nele:
+                raise ValueError(f"Number of elements can only be '{self.nele}'")
+        return out
+
+
+def uint(value, larger=-1):
+    val = int(value)
+    if val > larger:
+        return val
+    raise ValueError(f"Value '{val}' smaller than expected '{larger}'")
+
+
+def flatten_validator_dct(validators):
 
-    Returns
-    -------
-    dict
-        all validators in a single dictionary
-    """
     out = {}
-    for name, parser in base_validators.items():
-        parser_name = name.capitalize() + "Validator"
-        out[name] = type(parser_name, (ValidatorBase,), {'_parse': staticmethod(parser)})
-    for name, parser in range_validators.items():
-        parser_name = name.capitalize() + "Validator"
-        out[name] = type(parser_name, (RangeValidator,), {'_parse': staticmethod(parser)})
+    for validator_type in validators:
+        #
+        for typ, parser in validator_type.cases.items():
+            if typ in out:
+                raise ValueError(f"Validator Type '{typ}' already defined")
+            out[typ] = ValidatorParser(validator_type.cls, parser)
     return out
 
 
-class Validator:
-    """Factory class"""
+class ValidatorSelector:
 
-    parsers = create_validators(
-        {'str': str,
-         'bool': bool_parser,
-         'list': list_parser,
-         'ilist': ilist_parser,
-         'ilist_np': ilist_np_parser,
-         'flist': flist_parser,
-         'flist_np': flist_np_parser,
-         'file': abspath,  # return abspath
-         'folder': abspath,
-         'existing_file': file_exists,
-         'existing_folder': folder_exists,
-         'non_existing_file': non_existing_path,
-         'non_existing_folder': non_existing_path,
-         # python objects
-         'python(list)': as_python_list,
-         'python(dict)': as_python_dict,
-         'python(tuple)': as_python_tuple,
-         'python(np.array)': as_python_numpy_array,
-         }, {'int': int,
-             'float': float,
-             }
-    )
+    """Contains the logic how to select a particular validator"""
 
-    _validators = {'base': ValidatorBase, 'range': RangeValidator}
+    validators = flatten_validator_dct([
+        ValidatorType(DelayedDefaultValidator, {
+            'existing_file': file_exists,
+            'existing_folder': folder_exists,
+            'non_existing_file': non_existing_path,
+            'non_existing_folder': non_existing_path,
+        }),
+        #
+        ValidatorType(BaseValidator, {
+            'str': str,
+            'bool': bool_parser,
+            'list': list_parser,
+            'ilist': ilist_parser,
+            'ilist_np': ilist_np_parser,
+            'flist': flist_parser,
+            'flist_np': flist_np_parser,
+            'file': abspath,  # return abspath
+            'folder': abspath,
+            # python objects
+            'python(list)': as_python_list,
+            'python(dict)': as_python_dict,
+            'python(tuple)': as_python_tuple,
+            'python(np.array)': as_python_numpy_array,
+        }),
+        #
+        ValidatorType(RangeValidator, {
+            'int': int,
+            'float': float,
+        }),
+    ])
 
-    def __new__(cls, typ, default=NOT_DEFINED, choices=None):
-        if typ not in cls.parsers:
-            raise ValueError(f"Validator '{typ}' not defined")
-        return cls.parsers[typ](default=default, choices=choices)
+    types = {'base': BaseValidator, 'range': RangeValidator,
+             'delayed_default': DelayedDefaultValidator}
+
+    def __new__(cls, typ):
+        res = cls.validators.get(typ)
+        if res is None:
+            raise ValueError(f"Typ '{typ}' is unknown, use one of [{' '.join(cls.validators )}]")
+        return res
 
     @classmethod
     def add_validator(cls, name, func, typ='base'):
@@ -536,14 +622,101 @@ class Validator:
         ValueError
             In case the typ is unknown
         """
-        base_cls = cls._validators.get(typ, None)
-        if base_cls is None:
-            raise ValueError(f"Validator type '{typ}' not known")
-        parser_name = name.capitalize() + "Validator"
-        cls.parsers[name] = type(parser_name, (base_cls,), {'_parse': staticmethod(func)})
+        basetyp = cls.types.get(typ)
+        if basetyp is None:
+            raise ValueError("ValidatorBaseType  '{typ}' unknown")
+        if name in cls.validators:
+            raise ValueError(f"Validator type '{typ}' already known")
+        cls.validators[name] = ValidatorParser(basetyp, func)
 
     @classmethod
     def remove_validator(cls, name):
         """Remove validator """
-        if name in cls.parsers:
-            del cls.parsers[name]
+        del cls.validators[name]
+
+
+class Validator:
+
+    """ Validator Factory class """
+
+    def __new__(cls, typ, default=NOT_DEFINED, choices=None):
+        return cls._get_all_validators(typ, default, choices)
+
+    @classmethod
+    def add_validator(cls, name, func, *, typ='base'):
+        """Add a new custom validator.
+
+        Parameters
+        ----------
+        name: str
+            name of the validator typ
+        func: function
+            validation function, should raise ValueError on fail
+        typ: str, optional
+            typ of validator, currently: base, range
+
+        Raises
+        ------
+        ValueError
+            In case the typ is unknown
+        """
+        ValidatorSelector.add_validator(name, func, typ=typ)
+
+    @classmethod
+    def remove_validator(cls, name):
+        """Remove validator """
+        ValidatorSelector.remove_validator(name)
+
+    @classmethod
+    def overwrite_validator(cls, name, func, *, typ='base'):
+        """Add a new custom validator.
+
+        Parameters
+        ----------
+        name: str
+            name of the validator typ
+        func: function
+            validation function, should raise ValueError on fail
+        typ: str, optional
+            typ of validator, currently: base, range
+
+        Raises
+        ------
+        ValueError
+            In case the typ is unknown
+        """
+        cls.remove_validator(name)
+        cls.add_validator(name, func, typ=typ)
+
+    @classmethod
+    def _get_list_info(cls, typ):
+        """TODO: improve error messages"""
+        if typ[-1] != ')':
+            raise ValueError(f"Do not understand type '{typ}'")
+        typ = typ[5:-1]
+        typ = typ.split(':')
+        if len(typ) == 1:
+            return typ[0].strip(), ListInfo(True, -1)
+        if len(typ) > 2:
+            raise ValueError(f"Do not understand type '{typ}'")
+        try:
+            return typ[0].strip(), ListInfo(True, uint(typ[1], larger=0))
+        except ValueError:
+            raise ValueError(f"Do not understand type '{typ}'") from None
+
+    @classmethod
+    def _get_all_validators(cls, typ, default, choices):
+        # list(typ) or list(typ, 10) are special validators
+        if typ.startswith('list('):
+            typ, list_info = cls._get_list_info(typ)
+        else:
+            list_info = ListInfo(False, 0)
+        #
+        if typ == 'bool' and choices is None:
+            choices = 'y, n'
+        #
+        clstyp, func = ValidatorSelector(typ)
+        if list_info.is_list is True:
+            validator = clstyp(func, default=NOT_DEFINED, choices=choices)
+            return ListValidator(validator, list_info.nele, default=default)
+        return clstyp(func, default=default, choices=choices)
